#!/usr/bin/env cwltool
cwlVersion: v1.2
class: Workflow
requirements:
  StepInputExpressionRequirement: {}
  InlineJavascriptRequirement: {}
  MultipleInputFeatureRequirement: {}
  SubworkflowFeatureRequirement: {}
  ScatterFeatureRequirement: {}

label: nanopore workflow with direct FASTQ reads inputs (without Guppy)
doc: |
    Workflow for sequencing with ONT nanopore, from basecalling to assembly quality.
    Steps:
      - Kraken2 (taxonomic classification of FASTQ reads)
      - Krona (classification visualization)
      - Flye (de novo assembly)
      - Medaka (assembly polishing)
      - QUAST (assembly quality reports)

outputs:
  kraken2_output:
    label: Kraken2 reports
    doc: Kraken2 taxonomic classification reports
    type: Directory
    outputSource: kraken2_files_to_folder/results
  krona_output:
    label: Krona taxonomy visualization
    doc: Visual presentation in HTML of Kraken2 results
    type: Directory
    outputSource: krona_files_to_folder/results
  flye_output:
    label: Flye de novo assembler for single-molecule reads
    doc: Flye output directory
    type: Directory
    outputSource: flye_files_to_folder/results
  medaka_output:
    label: Medaka polisher
    doc: Polishing of Flye assembly
    type: Directory
    outputSource: medaka_files_to_folder/results
  quast_output:
    label: QUAlity assessment
    doc: QUAST analysis output
    type: Directory
    outputSource: quast_files_to_folder/results

inputs:
  # General
  identifier:
    type: string
    doc: Identifier for this dataset used in this workflow
    label: identifier used
  threads:
    type: int?
    doc: number of threads to use for computational processes
    label: number of threads
  nanopore_fastq_reads:
    type: File[]
    doc: file of FASTQ reads
    label: nanopore FASTQ reads
  # Kraken2
  kraken_database:
    type: Directory
    doc: full pathway with database location of kraken2
  # Medaka
  basecall_model:
    type: string
    doc: basecalling model used with Guppy
    label: basecalling model

steps:
############################################# INTEGRATED
#### merging of FASTQ files to only one for Kraken2
  concatenate: 
    # when: $(inputs.infiles != null)
    run: ../bash/concatenate.cwl
    in:
      infiles: 
        source: [nanopore_fastq_reads]
        linkMerge: merge_flattened
        pickValue: all_non_null
      outname: identifier
    out: [output]
#############################################
#### taxonomic classification of reads with Kraken2
  workflow_kraken2:
    label: Taxonomic classification with Kraken2
    doc: Taxonomic classification of FASTQ reads
    run: ../kraken2/kraken2.cwl
    in:
      identifier: identifier
      threads: threads
      nanopore: concatenate/output
      database: kraken_database
    out: [standard_report, sample_report]
#############################################
#### Visual representation of taxonomic classification
  workflow_krona:
    label: taxonomic classification visualization
    doc: visualization of taxonomic classification with Krona
    run: ../krona/krona.cwl
    in:
      kraken: workflow_kraken2/sample_report
    out: [krona_html]
#############################################
#### De novo assembly with Flye
  workflow_flye:
    label: de novo assembly
    doc: de novo assembly of single-molecule reads with Flye
    run: ../flye/flye.cwl
    in:
      nano_raw: concatenate/output
      threads: threads
    out: [00_assembly, 10_consensus, 20_repeat, 30_contigger, 40_polishing, assembly, assembly_info, flye_log, params]
#############################################
#### Polishing of assembled genome with Medaka
  workflow_medaka:
    label: polishing of assembly
    doc: Medaka for polishing of assembled genome
    run: ../medaka/medaka.cwl
    in:
      threads: threads
      draft_assembly: workflow_flye/assembly
      reads: concatenate/output
      basecall_model: basecall_model
    out: [polished_assembly, calls_to_draft, gaps_in_draft_coords, probs]
#############################################
#### Assembly evaluation with QUAST
  workflow_quast:
    label: assembly evaluation
    doc: evaluation of polished assembly qith QUAST
    run: ../quast/quast.cwl
    in:
      assembly: workflow_medaka/polished_assembly
    out: [basicStats, icarusDir, icarusHtml, quastReport, quastLog, transposedReport]

#############################################
#############################################    
#### Move to folder if not part of a workflow
  kraken2_files_to_folder:
    doc: Preparation of Kraken2 output files to a specific output folder
    label: Kraken2 output folder
    run: ../expressions/files_to_folder.cwl
    in:
      files:
        source: [workflow_kraken2/standard_report, workflow_kraken2/sample_report]
        linkMerge: merge_flattened
      destination:
        valueFrom: $("1_Kraken2_classification")
    out:
      [results]
#############################################
#### Move to folder if not part of a workflow
  krona_files_to_folder:
    doc: Preparation of Krona output files to a specific output folder
    label: Krona output folder
    run: ../expressions/files_to_folder.cwl
    in:
      files:
        source: [workflow_krona/krona_html]
        linkMerge: merge_flattened
      destination:
        valueFrom: $("2_Krona_taxa_visualization")
    out:
      [results]
#############################################
#### Move to folder if not part of a workflow
  flye_files_to_folder:
    doc: Preparation of Flye output files to a specific output folder
    label: Flye output folder
    run: ../expressions/files_to_folder.cwl
    in:
      files:
        source: [workflow_flye/assembly, workflow_flye/assembly_info, workflow_flye/flye_log, workflow_flye/params]
        linkMerge: merge_flattened
      # folders:
        # source: [workflow_flye/00_assembly, workflow_flye/10_consensus, workflow_flye/20_repeat, workflow_flye/30_contigger, workflow_flye/40_polishing]
        # linkMerge: merge_flattened
      destination:
        valueFrom: $("3_Flye_denovo_assembly")
    out:
      [results]
#############################################
#### Move to folder if not part of a workflow
  medaka_files_to_folder:
    doc: Preparation of Medaka output files to a specific output folder
    label: Medaka output folder
    run: ../expressions/files_to_folder.cwl
    in:
      files:
        source: [workflow_medaka/polished_assembly, workflow_medaka/calls_to_draft, workflow_medaka/gaps_in_draft_coords, workflow_medaka/probs]
        linkMerge: merge_flattened
      destination:
        valueFrom: $("4_Medaka_assembly_polishing")
    out:
      [results]
#############################################
#### Move to folder if not part of a workflow
  quast_files_to_folder:
    doc: Preparation of QUAST output files to a specific output folder
    label: QUAST output folder
    run: ../expressions/files_to_folder.cwl
    in:
      files: 
        source: [workflow_quast/quastLog, workflow_quast/icarusHtml, workflow_quast/quastReport, workflow_quast/transposedReport]
        linkMerge: merge_flattened
      folders:
        source: [workflow_quast/basicStats, workflow_quast/icarusDir]
        linkMerge: merge_flattened
      destination:
        valueFrom: $("5_QUAST_assembly_validation")
    out:
      [results]

s:author:
  - class: s:Person
    s:identifier: https://orcid.org/0000-0002-5516-8391
    s:email: mailto:german.royvalgarcia@wur.nl
    s:name: Germán Royval
  - class: s:Person
    s:identifier: https://orcid.org/0000-0001-8172-8981
    s:email: mailto:jasper.koehorst@wur.nl
    s:name: Jasper Koehorst
  - class: s:Person
    s:identifier: https://orcid.org/0000-0001-9524-5964
    s:email: mailto:bart.nijsse@wur.nl
    s:name: Bart Nijsse

s:citation: https://m-unlock.nl
s:codeRepository: https://gitlab.com/m-unlock/cwl
s:dateCreated: "2021-12-23"
s:license: https://spdx.org/licenses/Apache-2.0 
s:copyrightHolder: "UNLOCK - Unlocking Microbial Potential"

$namespaces:
  s: https://schema.org/
  edam: http://edamontology.org/

$schemas:
 - http://edamontology.org/EDAM_1.18.owl
